/*
 * Include header files for all drivers that have been imported from
 * Atmel Software Framework (ASF).
 */
/*
 * Support and FAQ: visit <a href="https://www.microchip.com/support/">Microchip Support</a>
 */
#include <asf.h>
#include <stdio.h>
#include <avr/boot.h>

#include "conf_clock.h"


static int uart0_tx(char c, FILE *stream)
{
	if(c == '\n')
	{
		while((UCSR0A & (1 << UDRE0)) == 0);
		UDR0 = 0x0D;	// carriage return
	}

	while((UCSR0A & (1 << UDRE0)) == 0);
	UDR0 = c;


	return 0;
}



static void uart0_rx(FILE *stream)
{
	char data;

	while (!(UCSR0A & (1 << RXC0)));

	data = UDR0;
}



static FILE mystdio = FDEV_SETUP_STREAM(uart0_tx, uart0_rx, _FDEV_SETUP_RW);

void USART0_Printf_Init(uint32_t baudrate)
{
	float f_value;
	uint32_t ubr_reg;

	sysclk_enable_peripheral_clock(&UCSR0A);

	/* [Caution] Set baud rate : Expression BOARD_CLOCK / (16 * baudrate) - 1 may cause data crack. */
	f_value = ((float)BOARD_EXTERNAL_CLK / (16. * baudrate) - 1.);

	ubr_reg = (uint32_t)f_value;
	if(f_value - ubr_reg > 0.5)
		ubr_reg++;

	UCSR0A = 0;
	UCSR0B = 0;

	/* Set frame format: 8data, 1stop bit */
	UCSR0C = (0 << USBS0) | (3 << UCSZ00);

	UBRR0H = (unsigned char)((ubr_reg >> 8) & 0xFF);
	UBRR0L = (unsigned char)(ubr_reg & 0xFF);

	/* Enable receiver and transmitter #1. */
	UCSR0B = (1 << RXEN0) | (1 << TXEN0);

	// Assign device(mystdio) to stdout & stdin.
	stdout = stdin = &mystdio;
}



int main(void)
{
	uint32_t baudrate = 115200;
	uint16_t address;
	uint8_t data;
	uint8_t *pverify;


	/* Insert system clock initialization code. */
	sysclk_init();

	board_init();

	USART0_Printf_Init(baudrate);

	delay_ms(100);

	printf("[ATmega168PB] : CPU Clock = %ld Hz\r\n", BOARD_EXTERNAL_CLK);

	cli();

	address = 0x1800;
	data = 0xAB;

	boot_page_erase(address);
	boot_page_fill(address, data);
	boot_page_write(address);

	pverify = (uint8_t *)address;
	printf("data = 0x%X\r\n", *pverify);

	address = 0x3000;
	data = 0xCD;

	boot_page_erase(address);
	boot_page_fill(address, data);
	boot_page_write(address);

	pverify = (uint8_t *)address;
	printf("data = 0x%X\r\n", *pverify);

	sei();
	
	while(1);
}
